@@ -3,9 +3,17 @@ import inspect
 import json
 import types
 from collections import deque
-from dataclasses import asdict, field, fields, is_dataclass, make_dataclass, _get_field
+from dataclasses import (
+    asdict,
+    field,
+    Field,
+    fields,
+    is_dataclass,
+    make_dataclass,
+    _get_field,
+)
 from datetime import datetime
-from typing import Any, Optional
+from typing import Any, Iterable, Optional
 
 from supermodels.exceptions import ImmutableFieldError, ValidationError
 from supermodels.fields import (
@@ -117,19 +125,32 @@ def __str__(self) -> str:
         return repr(self)
 
 
-def get_supermodel_fields(
-    cls, versioned: bool, timestamped: bool
-) -> list[tuple[str, type, field]]:
-    """Returns the fields of the supermodel.
-
-    These include the underlying dataclass fields
-    plus extra version and timestamp fields
-    if required per the supermodel options.
-    """
+def get_supermodel_fields(cls) -> list[tuple[str, type, field]]:
+    """Returns the dataclass fields as field definitions for use in make_dataclass."""
     annotations = cls.__dict__.get("__annotations__", {})
 
     attrs = []
 
+    for name, annotation in annotations.items():
+        field_def = getattr(cls, name, None)
+        kw_only = getattr(field_def, "kw_only", False) if field_def else False
+        attrs.append((name, annotation, _get_field(cls, name, annotation, kw_only)))
+
+    return attrs
+
+
+def extend_supermodel_fields(
+    field_defs: Iterable[tuple[str, type, field]], versioned: bool, timestamped: bool
+) -> list[tuple[str, type, field]]:
+    """Returns the dataclass field definitions extended with supermodel field definitions.
+
+    These include the underlying dataclass fields
+    plus extra version and timestamp fields
+    if required per the supermodel arguments.
+    """
+    attrs = list(field_defs)
+    annotations = {attr[0]: attr for attr in attrs}
+
     if timestamped:
         for attr in ("_created", "_updated"):
             if attr not in annotations:
@@ -138,11 +159,6 @@ def get_supermodel_fields(
     if versioned and "_version" not in annotations:
         attrs.append(("_version", int, version_field()))
 
-    for name, annotation in cls.__dict__.get("__annotations__", {}).items():
-        field_def = getattr(cls, name, None)
-        kw_only = getattr(field_def, "kw_only", False) if field_def else False
-        attrs.append((name, annotation, _get_field(cls, name, annotation, kw_only)))
-
     return attrs
 
 
@@ -312,47 +328,107 @@ def update(self, **kwargs):
     return type(self)(**data)
 
 
-def supermodel(cls=None, **kwargs):
-    """Provides the supermodel decorator for dataclasses."""
+def make_supermodel(
+    cls_name: str,
+    field_defs: Iterable[str | tuple[str, type] | tuple[str, type, Field]],
+    bases: tuple[type, ...] = ...,
+    namespace: dict[str, Any] | None = ...,
+    init: bool = True,
+    repr: bool = True,
+    eq: bool = True,
+    order: bool = False,
+    unsafe_hash: bool = False,
+    frozen: bool = True,
+    match_args: bool = True,
+    kw_only: bool = False,
+    slots: bool = False,
+    versioned: bool = False,
+    timestamped: bool = False,
+) -> type:
+    """Returns a new dynamically created supermodel."""
+    namespace.update(
+        dict(
+            __post_init__=__post_init__,
+            __help__=classmethod(__help__),
+            __type__=property(__type__),
+            __str__=__str__,
+            update=update,
+            from_dict=classmethod(from_dict),
+            from_json=classmethod(from_json),
+            to_dict=to_dict,
+            to_json=to_json,
+            raise_validation_error=raise_validation_error,
+            _SUPERMODEL=True,
+            _VERSIONED=versioned,
+            _TIMESTAMPED=timestamped,
+        )
+    )
+
+    field_defs = extend_supermodel_fields(field_defs, versioned, timestamped)
+
+    return make_dataclass(
+        cls_name,
+        bases=bases,
+        fields=field_defs,
+        namespace=namespace,
+        init=init,
+        repr=repr,
+        eq=eq,
+        order=order,
+        unsafe_hash=unsafe_hash,
+        frozen=frozen,
+        match_args=match_args,
+        kw_only=kw_only,
+        slots=slots,
+    )
+
+
+def supermodel(
+    cls=None,
+    init: bool = True,
+    repr: bool = True,
+    eq: bool = True,
+    order: bool = False,
+    unsafe_hash: bool = False,
+    frozen: bool = True,
+    match_args: bool = True,
+    kw_only: bool = False,
+    slots: bool = False,
+    versioned: bool = False,
+    timestamped: bool = False,
+):
+    """Provides a decorator method that returns a supermodel dataclass.
+
+    The returned supermodel attaches extra methods and attributes
+    to the provided class based on the fields defined in the class
+    as well as the arguments passed to this method.
+    """
 
     def wrapper(cls):
-        frozen = kwargs.pop("frozen", True)
-        versioned = kwargs.pop("versioned", False)
-        timestamped = kwargs.pop("timestamped", False)
-
-        namespace = {member[0]: member[1] for member in get_class_members(cls)}
-        namespace.update(
-            dict(
-                __post_init__=__post_init__,
-                __help__=classmethod(__help__),
-                __type__=property(__type__),
-                __qualified_type__=property(get_qualified_type(cls)),
-                __str__=__str__,
-                update=update,
-                from_dict=classmethod(from_dict),
-                from_json=classmethod(from_json),
-                to_dict=to_dict,
-                to_json=to_json,
-                raise_validation_error=raise_validation_error,
-                _SUPERMODEL=True,
-                _VERSIONED=versioned,
-                _TIMESTAMPED=timestamped,
-            )
-        )
-
-        field_defs = get_supermodel_fields(cls, versioned, timestamped)
+        field_defs = get_supermodel_fields(cls)
 
         mro = inspect.getmro(cls)
         bases = tuple(c for c in mro if is_dataclass(c) and c != cls)
 
-        kwargs["frozen"] = frozen
+        namespace = {member[0]: member[1] for member in get_class_members(cls)}
+        namespace.update({"__qualified_type__": property(get_qualified_type(cls))})
 
-        return make_dataclass(
-            cls.__name__,
+        return make_supermodel(
+            cls_name=cls.__name__,
+            field_defs=field_defs,
             bases=bases,
-            fields=field_defs,
             namespace=namespace,
-            **kwargs,
+            init=init,
+            repr=repr,
+            eq=eq,
+            order=order,
+            unsafe_hash=unsafe_hash,
+            frozen=frozen,
+            match_args=match_args,
+            kw_only=kw_only,
+            slots=slots,
+            versioned=versioned,
+            timestamped=timestamped,
         )
 
     if cls is None:
